// Variable globale
var CARTE = document.getElementById("carte"),
    CONTENEUR = document.getElementById("conteneur"),
    IMAGE = new Image(),
    MARGESHORIZONTALES,
    MARGESVERTICALES,
    QUADRILLAGEX,
    QUADRILLAGEY,
    DAMIERX,
    DAMIERY;

//Fonction de conversion
function id2xy (id) {
    "use strict";
    var coords = id.split("|");
    coords = coords.map(function(x) { return parseInt(x); });
    return coords;
}

//WebSocket
var adresse = document.URL;
adresse = 'ws' + adresse.slice(4, adresse.lastIndexOf('\/')) + '/websocket';
var websocket = new WebSocket(adresse);

websocket.onmessage = function(event) {
    "use strict";
    var message = JSON.parse(event.data),
        mode = message.mode,
        contenu = message.contenu;
    if ( mode == "initialisation" ) {
        initialisation(contenu);
    }
};

//Initialisation de la carte et des creatures selon le message
function initialisation(plateau) {
    "use strict";
    DAMIERX = plateau.damierx;
    DAMIERY = plateau.damiery;
    IMAGE.addEventListener("load", function () {
        calculMarges();
        creationCamouflage(plateau.visibles);
        adapatationCamouflage();
        for ( let creature of plateau.creatures ) {
            creationCreature(creature);
        }        
    }, false);
    IMAGE.src = plateau.fichierdm;
    CARTE.src = plateau.fichierdm;
}

//Calcul des marges en pourcentage du div
function calculMarges() {
    "use strict";
    var carteRatio = CARTE.width/CARTE.height,
        imageRatio = IMAGE.width/IMAGE.height;

    if ( carteRatio > imageRatio ) {
        MARGESHORIZONTALES = (CARTE.width-CARTE.height*imageRatio)*50/CARTE.width;
        MARGESVERTICALES = 0;
        QUADRILLAGEX = (100-2*MARGESHORIZONTALES)/DAMIERX;
        QUADRILLAGEY = 100/DAMIERY;
    } else {
        MARGESVERTICALES = (CARTE.height-CARTE.width/imageRatio)*50/CARTE.height;
        MARGESHORIZONTALES = 0;
        QUADRILLAGEX = 100/DAMIERX;
        QUADRILLAGEY = (100-2*MARGESVERTICALES)/DAMIERY;
    }
}

//Adaptation des div de camouflage en fonction des marges
function adapatationCamouflage() {
    "use strict";
    var element;
    for ( let x=0; x<DAMIERX; x++ ) {
        for ( let y=0; y<DAMIERY; y++ ) {
            element = document.getElementById(x+"|"+y);
            element.style.top = (MARGESVERTICALES+y*QUADRILLAGEY)+"%";
            element.style.left = (MARGESHORIZONTALES+x*QUADRILLAGEX)+"%";
            element.style.height = QUADRILLAGEY+"%";
            element.style.width = QUADRILLAGEX+"%";
        }
    }
}

function drop(event) {
  	"use strict";
    event.preventDefault();
    var idCreature = event.dataTransfer.getData("text"),
        creature = document.getElementById(idCreature),
        destination = event.target;
    dpltCreature(creature,destination);
}

//Création des div de camouflage
function creationCamouflage(visibles) {
    "use strict";
    var element;
    for ( let x=0; x<DAMIERX; x++ ) {
        for ( let y=0; y<DAMIERY; y++ ) {
            element = document.createElement("div");
            element.id = x+"|"+y;
            element.classList.add("camouflage");
            if ( visibles.includes(element.id) ) {
                //On ne camoufle pas
            } else if ( (x+y)%2 === 0 ) {
                element.classList.add("rouge");
            } else {
                element.classList.add("bleu");
            }

            element.addEventListener("click", basculeVisibilite , false);
            element.addEventListener("dragover", function( event ) { event.preventDefault(); }, false);
            element.addEventListener("drop", drop, false);

            CONTENEUR.appendChild(element);
        }
    }
}


document.getElementById("rayon").value = 1;
document.getElementById("rayon").addEventListener("wheel", modifRayon, false);

function modifRayon(event) {
  	"use strict";
    event.preventDefault();
    var rayon = document.getElementById("rayon");
    if ( event.deltaY > 0 ) {
        rayon.value = parseInt(rayon.value) - 1;
    } else {
        rayon.value = parseInt(rayon.value) + 1;
    }
}

//Bascule de la visibilité du camouflage
function basculeVisibilite(event) {
    "use strict";
    var x,y,message,
        rayon = document.getElementById("rayon").value;
    [x,y] = id2xy(event.target.id);
    for ( let i=0; i<rayon; i++ ) {
        for ( let j=0; j<rayon; j++ ) {
            if ( (x+i+y+j)%2 === 0 ) {
                document.getElementById((x+i)+"|"+(y+j)).classList.toggle("rouge");
            } else {
                document.getElementById((x+i)+"|"+(y+j)).classList.toggle("bleu");
            }
            message = {
                "mode":"basculevisibilitecarte",
                "contenu":(x+i)+"|"+(y+j)
            };
            message = JSON.stringify(message);
            websocket.send(message);
        }
    }
}

//Ajout d'une créature par le formulaire
function ajoutCreature() {
    "use strict";
    var nom, taille, couleur, message,
        creature = {"nom":null, "coords":"0|0", "pv":0, "taille":null, "couleur":null, "visible":false};
    nom = document.getElementById("nom").value;
    creature.nom = nom;
    taille = document.getElementById("taille").value;
    creature.taille = taille;
    couleur = document.getElementById("couleur").value;
    creature.couleur = couleur;
    creationCreature(creature);
    message = {
        "mode":"ajoutcreature",
        "nom":nom,
        "taille":taille,
        "couleur":couleur
    };
    message = JSON.stringify(message);
    websocket.send(message);

}

document.getElementById("ajout").addEventListener("click", ajoutCreature, false);

//Vérification du nom de la créature
function verifNom() {
  	"use strict";
    var element = document.getElementById("nom");
    for ( let creature of document.getElementsByClassName("creature") ) {
        if ( element.value == creature.id ) {
            element.style.backgroundColor = "red";
            return;
        }
        element.style.backgroundColor = "green";
    }
}

document.getElementById("nom").addEventListener("input", verifNom, false);

//Création d'une créature
function creationCreature(creature) {
  	"use strict";
    var element, taille;
    element = document.createElement("div");
    element.id = creature.nom;
    element.innerHTML = creature.nom + "<br/>" + creature.pv;
    element.style.background = creature.couleur;
    element.classList.add("creature");
    element.setAttribute("draggable", "true");

    if ( creature.taille == "TP" ) { taille = "40%"; }
    else if ( creature.taille == "P" ) { taille = "90%"; }
    else if ( creature.taille == "M" ) { taille = "90%"; }
    else if ( creature.taille == "G" ) { taille = "190%"; }
    element.style.height = taille;
    element.style.width = taille;

    if ( !creature.visible ) { element.classList.add("invisible"); }

    element.addEventListener("click", modifVisibilite, false);
    element.addEventListener("wheel", modifPv, false);
    element.addEventListener("dblclick", suppressionDuneCreature , false);
    element.addEventListener("dragstart", function( event ) {
        event.dataTransfer.setData("text", event.target.id);
    }, false);
    
    document.getElementById(creature.coords).appendChild(element);
}

//Déplacement d'une créature
function dpltCreature(creature,destination) {
    "use strict";
    var message;
    creature.parentNode.removeChild(creature);
    destination.appendChild(creature);
    message = {
        "mode":"dpltcreature",
        "creature":creature.id,
        "destination":destination.id
    };
    message = JSON.stringify(message);
    websocket.send(message);
}

//Modification des pv d'une créature
function modifPv(event) {
    "use strict";
    event.preventDefault();
    var message,
        creature = event.target,
        nom = creature.innerHTML.split("<br>")[0],
        pv = parseInt(creature.innerHTML.split("<br>")[1]);
    if ( event.deltaY > 0 ) { pv -= 1; }
    else { pv += 1; }
    creature.innerHTML = nom + "<br/>" + pv;
    message = {
        "mode":"modifpv",
        "creature":creature.id,
        "pv":pv
    };
    message = JSON.stringify(message);
    websocket.send(message);
}

//Modification de la visibilité d'une créature
function modifVisibilite(event) {
    "use strict";
    event.stopPropagation();
    var message,
        creature = event.target;
    creature.classList.toggle("invisible");
    message = {
        "mode":"modifvisibilite",
        "creature":creature.id,
    };
    message = JSON.stringify(message);
    websocket.send(message);
}

//Suppression d'une créature
function suppressionDuneCreature() {
    this.parentNode.removeChild(this);
    var message = {
        "mode":"suppression",
        "creature":this.id,
    };
    message = JSON.stringify(message);
    websocket.send(message);
}

//Fonction de zoom
document.getElementById("zoom").value = 1;
document.getElementById("zoom").addEventListener("wheel", modifZoom, false);

function modifZoom(event) {
  	"use strict";
    event.preventDefault();
    var zoom = document.getElementById("zoom"),
        conteneur = document.getElementById("conteneur");
    if ( event.deltaY > 0 && zoom.value != "1") {
        zoom.value = parseInt(zoom.value) - 1;
    } else if ( event.deltaY < 0 && zoom.value != "5") {
        zoom.value = parseInt(zoom.value) + 1;
    }
    conteneur.style.height = zoom.value+"00%";
    conteneur.style.width = zoom.value+"00%";
}

//Sauvegarde du plateau
function sauver() {
  	"use strict";
    var message = {
        "mode":"sauvegarde"
    };
    message = JSON.stringify(message);
    websocket.send(message);
}
document.getElementById("sauvegarde").addEventListener("click", sauver, false);

//Evenement lors du redimensionnement de la fenetre
(function() {
    "use strict";
    window.addEventListener("resize", resizeThrottler, false);
    var resizeTimeout;
    function resizeThrottler() {
        // ignore resize events as long as an actualResizeHandler execution is in the queue
        if ( !resizeTimeout ) {
            resizeTimeout = setTimeout(function() {
                resizeTimeout = null;
                actualResizeHandler();
            // The actualResizeHandler will execute at a rate of 20fps Toutes les 50ms
            }, 50);
        }
    }

    function actualResizeHandler() {
        // handle the resize event
        calculMarges();
        adapatationCamouflage();
    }

}());
