// Variable globale
var CARTE = document.getElementById("carte"),
    CONTENEUR = document.getElementById("conteneur"),
    IMAGE = new Image(),
    MARGESHORIZONTALES,
    MARGESVERTICALES,
    QUADRILLAGEX,
    QUADRILLAGEY,
    DAMIERX,
    DAMIERY;

//WebSocket
var adresse = document.URL;
adresse = 'ws' + adresse.slice(4, adresse.lastIndexOf('\/')) + '/websocket';
var websocket = new WebSocket(adresse);

websocket.onmessage = function(event) {
    "use strict";
    var contenu, creature, destination,
        message = JSON.parse(event.data),
            mode = message.mode;
    if ( mode == "initialisation" ) {
        contenu = message.contenu;
        initialisation(contenu);
    } else if ( mode == "basculevisibilitecarte" ) {
        contenu = message.contenu;
        basculeVisibilite(contenu);
    } else if ( mode == "dpltcreature" ) {
        creature = message.creature;
        destination = message.destination;
        dpltCreature(creature,destination);
    } else if ( mode == "ajoutcreature" ) {
        var creature = {"nom":message.nom,
                        "coords":"0|0",
                        "pv":0,
                        "taille":message.taille,
                        "couleur":message.couleur,
                        "visible":false
                       };
        creationCreature(creature);
    } else if ( mode == "modifvisibilite" ) {
        creature = message.creature;
        modifVisibilite(creature);
    } else if ( mode == "suppression" ) {
        creature = message.creature;
        suppressionDuneCreature(creature);
    }
};

//Initialisation de la carte et des créatures selon le message
function initialisation(plateau) {
    "use strict";
    DAMIERX = plateau.damierx;
    DAMIERY = plateau.damiery;
    IMAGE.addEventListener("load", function () {
        calculMarges();
        creationCamouflage(plateau.visibles);
        adapatationCamouflage();
        for ( let creature of plateau.creatures ) {
            creationCreature(creature);
        }        
    }, false);
    IMAGE.src = plateau.fichierpj;
    CARTE.src = plateau.fichierpj;
}

//Calcul des marges en pourcentage du div
function calculMarges() {
    "use strict";
    var carteRatio = CARTE.width/CARTE.height,
        imageRatio = IMAGE.width/IMAGE.height;

    if ( carteRatio > imageRatio ) {
        MARGESHORIZONTALES = (CARTE.width-CARTE.height*imageRatio)*50/CARTE.width;
        MARGESVERTICALES = 0;
        QUADRILLAGEX = (100-2*MARGESHORIZONTALES)/DAMIERX;
        QUADRILLAGEY = 100/DAMIERY;
    } else {
        MARGESVERTICALES = (CARTE.height-CARTE.width/imageRatio)*50/CARTE.height;
        MARGESHORIZONTALES = 0;
        QUADRILLAGEX = 100/DAMIERX;
        QUADRILLAGEY = (100-2*MARGESVERTICALES)/DAMIERY;
    }
}

//Adaptation des div de camouflage en fonction des marges
function adapatationCamouflage() {
    "use strict";
    var element;
    for ( let x=0; x<DAMIERX; x++ ) {
        for ( let y=0; y<DAMIERY; y++ ) {
            element = document.getElementById(x+"|"+y);
            element.style.top = (MARGESVERTICALES+y*QUADRILLAGEY)+"%";
            element.style.left = (MARGESHORIZONTALES+x*QUADRILLAGEX)+"%";
            element.style.height = QUADRILLAGEY+"%";
            element.style.width = QUADRILLAGEX+"%";
        }
    }
}

//Création des div de camouflage
function creationCamouflage(visibles) {
    "use strict";
    var element;
    for ( let x=0; x<DAMIERX; x++ ) {
        for ( let y=0; y<DAMIERY; y++ ) {
            element = document.createElement("div");
            element.id = x+"|"+y;
            element.classList.add("camouflage");
            element.classList.add("gris");
            CONTENEUR.appendChild(element);
        }
    }
    for ( let coords of visibles ) {
        basculeVisibilite(coords);
    }
}

//Fonction de zoom
document.getElementById("zoomplus").addEventListener("click", zoomPlus, false);
document.getElementById("zoommoins").addEventListener("click", zoomMoins, false);

function zoomPlus(event) {
  	"use strict";
    event.preventDefault();
    var conteneur = document.getElementById("conteneur"),
        zoom = conteneur.style.height;
    if (zoom == "") {
        zoom = 2;
    } else if (zoom == "900%") {
        zoom = 9;
    } else {
        zoom = parseInt(zoom[0])+1;
    }
    console.log(zoom);
    conteneur.style.height = zoom + "00%";
    conteneur.style.width = zoom + "00%";
}

function zoomMoins(event) {
  	"use strict";
    event.preventDefault();
    var conteneur = document.getElementById("conteneur"),
        zoom = conteneur.style.height;
    if (zoom == "" || zoom == "100%") {
        zoom = 1;
    } else {
        zoom = parseInt(zoom[0])-1;
    }
    console.log(zoom);
    conteneur.style.height = zoom + "00%";
    conteneur.style.width = zoom + "00%";
}
//Bascule de la visibilité du camouflage
function basculeVisibilite(coords) {
    "use strict";
    var element = document.getElementById(coords);
    element.classList.toggle("gris");
}

//Création d'une créature
function creationCreature(creature) {
    "use strict";
    var element, nom, taille;
    element = document.createElement("div");
    nom = /([a-z|A-Z]*)[0-9]*/.exec(creature.nom)[1];
    element.id = creature.nom;
    element.innerHTML = nom;
    element.style.background = creature.couleur;
    element.classList.add("creature");

    if ( creature.taille == "TP" ) { taille = "40%"; }
    else if ( creature.taille == "P" ) { taille = "90%"; }
    else if ( creature.taille == "M" ) { taille = "90%"; }
    else if ( creature.taille == "G" ) { taille = "190%"; }
    element.style.height = taille;
    element.style.width = taille;

    if ( !creature.visible ) { element.classList.add("invisible"); }

    document.getElementById(creature.coords).appendChild(element);
}

//Déplacement d'une créature
function dpltCreature(creature,destination) {
    "use strict";
    creature = document.getElementById(creature);
    destination = document.getElementById(destination);
    creature.parentNode.removeChild( creature );
    destination.appendChild( creature );
}

//Modification de la visibilité d'une créature
function modifVisibilite(creature) {
    "use strict";
    creature = document.getElementById(creature);
    creature.classList.toggle("invisible");
}

//Suppression d'une créature
function suppressionDuneCreature(creature) {
    "use strict";
    creature = document.getElementById(creature);
    creature.parentNode.removeChild(creature);
}

//Evenement lors du redimensionnement de la fenetre
(function() {
    "use strict";
    window.addEventListener("resize", resizeThrottler, false);
    var resizeTimeout;
    function resizeThrottler() {
        // ignore resize events as long as an actualResizeHandler execution is in the queue
        if ( !resizeTimeout ) {
            resizeTimeout = setTimeout(function() {
                resizeTimeout = null;
                actualResizeHandler();
            // The actualResizeHandler will execute at a rate of 20fps Toutes les 50ms
            }, 50);
        }
    }

    function actualResizeHandler() {
        // handle the resize event
        calculMarges();
        adapatationCamouflage();
    }

}());
